import 'dart:convert';

import 'package:equatable/equatable.dart';

/// TenantLoginSettings model
class TenantLoginSettings extends Equatable {
  const TenantLoginSettings({
    required this.tenantName,
    required this.cookieDomain,
    required this.changePasswordSettings,
    required this.rememberMeSettings,
    required this.loginProviders,
    required this.authenticationTypes,
    required this.tenantDomain,
  });

  final String tenantName;
  final String cookieDomain;
  final ChangePasswordSettings changePasswordSettings;
  final RememberMeSettings rememberMeSettings;
  final LoginProviders loginProviders;
  final List<String> authenticationTypes;
  final String tenantDomain;

  @override
  List<Object?> get props => [
        tenantName,
        cookieDomain,
        changePasswordSettings,
        rememberMeSettings,
        loginProviders,
        authenticationTypes,
        tenantDomain
      ];

  Map<String, dynamic> toMap() {
    return {
      'tenantName': tenantName,
      'cookieDomain': cookieDomain,
      'changePasswordSettings': changePasswordSettings.toMap(),
      'rememberMeSettings': rememberMeSettings.toMap(),
      'loginProviders': loginProviders.toMap(),
      'authenticationTypes': authenticationTypes,
      'tenantDomain': tenantDomain,
    };
  }

  factory TenantLoginSettings.fromMap(Map<String, dynamic> map) {
    return TenantLoginSettings(
      tenantName: map['tenantName'],
      cookieDomain: map['cookieDomain'],
      changePasswordSettings:
          ChangePasswordSettings.fromMap(map['changePasswordSettings']),
      rememberMeSettings: RememberMeSettings.fromMap(map['rememberMeSettings']),
      loginProviders: LoginProviders.fromMap(map['loginProviders']),
      authenticationTypes: List.from(map['authenticationTypes']),
      tenantDomain: map['tenantDomain'],
    );
  }

  String toJson() => json.encode(toMap());

  factory TenantLoginSettings.fromJson(String source) =>
      TenantLoginSettings.fromMap(json.decode(source));
}

class ChangePasswordSettings extends Equatable {
  const ChangePasswordSettings({
    required this.allowed,
    this.url,
    this.customRecaptchaSiteKey,
  });

  final bool allowed;
  final String? url;
  final String? customRecaptchaSiteKey;

  @override
  List<Object?> get props => [allowed, url, customRecaptchaSiteKey];

  Map<String, dynamic> toMap() {
    return {
      'allowed': allowed,
      'url': url,
      'customRecaptchaSiteKey': customRecaptchaSiteKey,
    };
  }

  factory ChangePasswordSettings.fromMap(Map<String, dynamic> map) {
    return ChangePasswordSettings(
      allowed: map['allowed'],
      url: map['url'],
      customRecaptchaSiteKey: map['customRecaptchaSiteKey'],
    );
  }

  String toJson() => json.encode(toMap());

  factory ChangePasswordSettings.fromJson(String source) =>
      ChangePasswordSettings.fromMap(json.decode(source));
}

class RememberMeSettings extends Equatable {
  const RememberMeSettings({
    required this.allowed,
    this.expiration,
  });

  final bool allowed;
  final int? expiration;

  @override
  List<Object?> get props => [allowed, expiration];

  Map<String, dynamic> toMap() {
    return {
      'allowed': allowed,
      'expiration': expiration,
    };
  }

  factory RememberMeSettings.fromMap(Map<String, dynamic> map) {
    return RememberMeSettings(
      allowed: map['allowed'],
      expiration: map['expiration'],
    );
  }

  String toJson() => json.encode(toMap());

  factory RememberMeSettings.fromJson(String source) =>
      RememberMeSettings.fromMap(json.decode(source));
}

class LoginProviders extends Equatable {
  const LoginProviders({
    this.saml,
  });

  final SamlProvider? saml;

  @override
  List<Object?> get props => [saml];

  Map<String, dynamic> toMap() {
    return {
      'saml': saml?.toMap(),
    };
  }

  factory LoginProviders.fromMap(Map<String, dynamic> map) {
    return LoginProviders(
      saml: map['saml'] != null ? SamlProvider.fromMap(map['saml']) : null,
    );
  }

  String toJson() => json.encode(toMap());

  factory LoginProviders.fromJson(String source) =>
      LoginProviders.fromMap(json.decode(source));
}

class SamlProvider extends Equatable {
  const SamlProvider({
    this.url,
  });

  final String? url;

  @override
  List<Object?> get props => [url];

  SamlProvider copyWith({
    String? url,
  }) {
    return SamlProvider(
      url: url ?? this.url,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'url': url,
    };
  }

  factory SamlProvider.fromMap(Map<String, dynamic> map) {
    return SamlProvider(
      url: map['url'],
    );
  }

  String toJson() => json.encode(toMap());

  factory SamlProvider.fromJson(String source) =>
      SamlProvider.fromMap(json.decode(source));
}
