import 'dart:async';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:senior_authentication/senior_authentication.dart';
import 'package:senior_authentication/src/screens/password/bloc/password_form_state.dart';
import 'package:senior_authentication/src/utils/exceptions.dart';

part 'password_event.dart';
part 'password_state.dart';

class PasswordBloc extends Bloc<PasswordEvent, PasswordState> {
  PasswordBloc({required this.authenticationRepository, required this.username})
      : super(PasswordState(username: username));

  final AuthenticationRepository authenticationRepository;
  final String username;

  @override
  Stream<PasswordState> mapEventToState(
    PasswordEvent event,
  ) async* {
    if (event is PasswordVisibilityChange) {
      yield state.copyWith(passwordVisible: !state.passwordVisible);
    }

    if (event is PasswordChanged) {
      yield state.copyWith(
          password: event.password, formState: const PasswordFormInitial());
    }

    if (event is PasswordSubmitted) {
      yield state.copyWith(formState: PasswordFormLoadInProgress());

      try {
        var userLogin =
            UserLogin(username: state.username, password: state.password);

        await authenticationRepository.logInWithEmailAndPassword(userLogin);

        yield state.copyWith(formState: PasswordFormLoadSuccess());
      } on LogInWithEmailAndPasswordIncorrect catch (_) {
        yield state.copyWith(formState: PasswordFormIncorrectPassword());
      } catch (_) {
        yield state.copyWith(
            formState:
                PasswordFormLoadFailure(Exception('Erro na autenticação')));
      }
    }
  }
}
