import 'package:dio/dio.dart';
import 'package:senior_authentication/src/models/constants/regex.dart';
import 'package:senior_authentication/src/utils/preference_storage.dart';

class AuthenticationUtils {
  static String parseTokenValue(String token) {
    var splitted = token.split(' ');
    if (splitted.length > 1) {
      return token;
    } else {
      return 'Bearer $token';
    }
  }
}

class PlatformInterceptor {
  Dio addInterceptor(String bridgeUrl, Dio dio) {
    dio.interceptors
        .add(InterceptorsWrapper(onRequest: (options, handler) async {
      var sendToBridge = options.path.contains(dontUseHttp);
      var includeBar = !options.path.startsWith('/');
      var token = await Storage.getToken();

      if (token.isNotEmpty && token.access_token.isNotEmpty) {
        options.headers = {
          ...options.headers,
          'Authorization':
              AuthenticationUtils.parseTokenValue(token.access_token)
        };
      }

      if (sendToBridge) {
        options.path = includeBar
            ? '$bridgeUrl/${options.path}'
            : '$bridgeUrl${options.path}';
      }

      handler.next(options);
    }));

    return dio;
  }
}
